﻿using System;
using Windows.ApplicationModel.AppService;
using Windows.ApplicationModel.Background;
using Windows.Devices;
using Windows.Devices.Gpio;
using Windows.Foundation.Collections;
using Microsoft.IoT.Lightning.Providers;

namespace BgApp
{
	public sealed class StartupTask : IBackgroundTask
	{
		private BackgroundTaskDeferral deferral = null;
		private AppServiceConnection appcon = null;

		private GpioPin pin17 = null, pin18 = null;

		public void Run(IBackgroundTaskInstance taskInstance)
		{
			// Keep the background task
			deferral = taskInstance.GetDeferral();
			taskInstance.Canceled += OnTaskCanceled;

			var triggerDetails = taskInstance.TriggerDetails as AppServiceTriggerDetails;
			appcon = triggerDetails.AppServiceConnection;
			appcon.RequestReceived += OnConnectionReceived;

			// Initialize the pins by direct memory mapping.
			if (LightningProvider.IsLightningEnabled) {
				LowLevelDevicesController.DefaultProvider = LightningProvider.GetAggregateProvider();
			}

			GpioController gc = GpioController.GetDefault();

			pin17 = gc.OpenPin(17);
			pin17.SetDriveMode(GpioPinDriveMode.Output);

			pin18 = gc.OpenPin(18);
			pin18.SetDriveMode(GpioPinDriveMode.Input);
			pin18.ValueChanged += OnPin18Changed;
		}

		private async void OnPin18Changed(GpioPin sender, GpioPinValueChangedEventArgs args)
		{
			var messages = new ValueSet();
			if (pin18.Read() == GpioPinValue.High) {
				messages.Add("pin", true);
			} else {
				messages.Add("pin", false);
			}
			await appcon.SendMessageAsync(messages);
		}

		private void OnTaskCanceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason reason)
		{
			if (deferral != null) {
				deferral.Complete();
				deferral = null;
			}
		}

		private async void OnConnectionReceived(AppServiceConnection sender, AppServiceRequestReceivedEventArgs args)
		{
			var deferral = args.GetDeferral();

			ValueSet resp = new ValueSet();
			try {
				bool led = (bool)args.Request.Message["led"];
				if (pin17 != null) {
					pin17.Write(led ? GpioPinValue.High : GpioPinValue.Low);
					resp.Add("message", "ok");
				}else {
					resp.Add("message", "pin17 is not initialized.");
				}
			} catch (Exception ex) {
				resp.Add("message", ex.Message);
			}
			await args.Request.SendResponseAsync(resp);

			deferral.Complete();
		}
	}
}
