﻿using System;
using Windows.ApplicationModel.AppService;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace UiApp
{
	public sealed partial class MainPage : Page
	{
		public MainPage()
		{
			this.InitializeComponent();
		}

		private static readonly string ServiceName = "BgApp";
		private AppServiceConnection service = null;

		protected override async void OnNavigatedTo(NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);

			var listing = await AppServiceCatalog.FindAppServiceProvidersAsync(ServiceName);
			if (listing.Count != 1) return;

			service = new AppServiceConnection();
			service.AppServiceName = ServiceName;
			service.PackageFamilyName = listing[0].PackageFamilyName;

			var res = await service.OpenAsync();
			if (res != AppServiceConnectionStatus.Success) {
				txtStatus.Text = "Can't connect to " + res.ToString();
			} else {
				service.RequestReceived += OnServiceRequestReceived;
				txtStatus.Text = "Ready";
			}
		}

		private SolidColorBrush brush_on = new SolidColorBrush(Color.FromArgb(255, 209, 236, 255));
		private SolidColorBrush brush_off = new SolidColorBrush(Color.FromArgb(255, 255, 170, 204));

		private async void OnServiceRequestReceived(AppServiceConnection sender, AppServiceRequestReceivedEventArgs args)
		{
			await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, async () => {
				var d = args.GetDeferral();

				ValueSet res = new ValueSet();

				string lblmsg = "";
				SolidColorBrush bgbrush = new SolidColorBrush();
				try {
					bool pin = (bool)args.Request.Message["pin"];
					if(pin == true) {
						lblmsg = "Switch On";
						bgbrush = brush_on;
					} else {
						lblmsg = "Switch Off";
						bgbrush = brush_off;
					}
					res.Add("message", "ok");
				} catch (Exception ex) {
					res.Add("message", ex.Message);
				}
				await args.Request.SendResponseAsync(res);

				lblSwitch.Text = lblmsg;
				pnlSwitch.Background = bgbrush;

				d.Complete();
			});
		}

		private bool led_on = false;

		private async void OnLedButtonClick(object sender, Windows.UI.Xaml.RoutedEventArgs e)
		{
			SolidColorBrush fgbrush = new SolidColorBrush(Colors.Black);

			led_on = !led_on;

			ValueSet message = new ValueSet();
			message.Add("led", led_on);

			var resp = await service.SendMessageAsync(message);
			string msg;
			if (resp.Status == AppServiceResponseStatus.Success) {
				msg = resp.Message["message"].ToString();
			} else {
				msg = "An error has occured : " + resp.Status;
				fgbrush = new SolidColorBrush(Colors.Red);
			}

			await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () => {
				btnLED.Content = string.Format("LED {0}", (led_on == true) ? "On" : "Off");
				btnLED.Background = (led_on == true) ? brush_on : brush_off;
				txtStatus.Text = msg;
				txtStatus.Foreground = fgbrush;
			});
		}
	}
}
